from json          import loads
from pytest        import mark
from urllib.parse  import unquote

from paddle_billing_python_sdk.Environment      import Environment
from paddle_billing_python_sdk.FiltersUndefined import FiltersUndefined

from paddle_billing_python_sdk.Entities.Product                       import Product
from paddle_billing_python_sdk.Entities.ProductWithIncludes           import ProductWithIncludes
from paddle_billing_python_sdk.Entities.Collections.ProductCollection import ProductCollection
from paddle_billing_python_sdk.Entities.Shared.CustomData             import CustomData
from paddle_billing_python_sdk.Entities.Shared.TaxCategory            import TaxCategory

from paddle_billing_python_sdk.Resources.Events.Operations.ListEvents      import ListEvents
from paddle_billing_python_sdk.Resources.Products.Operations.CreateProduct import CreateProduct
from paddle_billing_python_sdk.Resources.Products.Operations.UpdateProduct import UpdateProduct
from paddle_billing_python_sdk.Resources.Shared.Operations.List.Pager      import Pager

from tests.Utils.TestClient   import mock_requests, test_client
from tests.Utils.ReadsFixture import ReadsFixtures


class TestProductsClient:
    @mark.parametrize(
        'operation, expected_request_body, expected_response_status, expected_response_body, expected_url',
        [
            (
                    CreateProduct(
                        name         = 'ChatApp Basic',
                        tax_category = TaxCategory.Standard,
                    ),
                    ReadsFixtures.read_raw_json_fixture('request/create_basic'),
                    200,
                    ReadsFixtures.read_raw_json_fixture('response/minimal_entity'),
                    f"{Environment.SANDBOX.base_url}/products",
            ),
            (
                    CreateProduct(
                        name         = 'ChatApp Full',
                        tax_category = TaxCategory.Standard,
                        description  = 'Spend more time engaging with students with ChataApp Education.',
                        image_url    = 'https://paddle-sandbox.s3.amazonaws.com/user/10889/2nmP8MQSret0aWeDemRw_icon1.png',
                        custom_data  = CustomData({
                            'features': {
                                'reports':        True,
                                'crm':            False,
                                'data_retention': True,
                            },
                        }),
                    ),
                    ReadsFixtures.read_raw_json_fixture('request/create_full'),
                    200,
                    ReadsFixtures.read_raw_json_fixture('response/full_entity'),
                    f"{Environment.SANDBOX.base_url}/products"
            ),
        ],
        ids = [
            "Create product with basic data",
            "Create product with full data",
        ],
    )
    def test_it_uses_expected_payload_on_create(
        self,
        test_client,
        mock_requests,
        operation,
        expected_request_body,
        expected_response_status,
        expected_response_body,
        expected_url
    ):
        mock_requests.post(
            url         = expected_url,
            status_code = expected_response_status,
            text        = expected_response_body
        )

        request_json = test_client.client.serialize_json_payload(
            FiltersUndefined.filter_undefined_values(operation.get_parameters())
        )

        response     = test_client.client.products.create(operation)
        last_request = mock_requests.last_request

        assert isinstance(response, Product)
        assert last_request is not None
        assert last_request.method       == 'POST'
        assert unquote(last_request.url) == expected_url, \
            "The URL does not match the expected URL, verify the Pagination query string is correct"
        assert loads(request_json)       == loads(expected_request_body), \
            "The request JSON generated by Client() doesn't match the expected fixture JSON"



    @mark.parametrize(
        'operation, expected_request_body, expected_response_status, expected_response_body, expected_url',
        [
            (
                    UpdateProduct(name='ChatApp Pro'),
                    ReadsFixtures.read_raw_json_fixture('request/update_single'),
                    200,
                    ReadsFixtures.read_raw_json_fixture('response/full_entity'),
                    f"{Environment.SANDBOX.base_url}/products/pro_01h7zcgmdc6tmwtjehp3sh7azf",
            ),
            (
                    UpdateProduct(name='ChatApp Pro', tax_category=TaxCategory.Saas),
                    ReadsFixtures.read_raw_json_fixture('request/update_partial'),
                    200,
                    ReadsFixtures.read_raw_json_fixture('response/full_entity'),
                    f"{Environment.SANDBOX.base_url}/products/pro_01h7zcgmdc6tmwtjehp3sh7azf",
            ),
            (
                    UpdateProduct(
                        name         = 'ChatApp Pro',
                        tax_category = TaxCategory.Saas,
                        description  = 'Spend more time engaging with students with ChatApp Pro.',
                        image_url    = 'https://paddle-sandbox.s3.amazonaws.com/pro.png',
                        custom_data  = CustomData({
                            'features': {
                                'reports':        True,
                                'crm':            True,
                                'data_retention': True,
                            },
                        }),
                    ),
                    ReadsFixtures.read_raw_json_fixture('request/update_full'),
                    200,
                    ReadsFixtures.read_raw_json_fixture('response/full_entity'),
                    f"{Environment.SANDBOX.base_url}/products/pro_01h7zcgmdc6tmwtjehp3sh7azf",
            ),
        ],
        ids = [
            "Update product with single new value",
            "Update product with partial new values",
            "Update product with completely new values",
        ],
    )
    def test_it_uses_expected_payload_on_update(
        self,
        test_client,
        mock_requests,
        operation,
        expected_request_body,
        expected_response_status,
        expected_response_body,
        expected_url
    ):
        mock_requests.patch(
            url         = expected_url,
            status_code = expected_response_status,
            text        = expected_response_body
        )

        request_json = test_client.client.serialize_json_payload(
            FiltersUndefined.filter_undefined_values(operation.get_parameters())
        )

        response     = test_client.client.products.update('pro_01h7zcgmdc6tmwtjehp3sh7azf', operation)
        last_request = mock_requests.last_request

        assert isinstance(response, Product)
        assert last_request is not None
        assert last_request.method       == 'PATCH'
        assert unquote(last_request.url) == expected_url, \
            "The URL does not match the expected URL, verify the Pagination query string is correct"
        assert loads(request_json)       == loads(expected_request_body), \
            "The request JSON generated by Client() doesn't match the expected fixture JSON"
